{%- macro field_properties(name="", type="") %}
sp.FieldProperties(
    ## Required
    name="{{ name }}",
    type="{{ type }}",
    ## Optional
    # title="",
    # format="",
    # description="",
    # example="",
    # categories=[],
    # categories_ordered=False,
),
{%- endmacro -%}
