import seedcase_sprout as sp

# from .resource_properties import resource_properties

package_properties = sp.PackageProperties(
    ## Required:
    name="{{ properties.name }}",
    title="",
    description="",
    licenses=[
        sp.LicenseProperties(
            ## Required:
            name="",
            ## Optional:
            # path="",
            # title="",
        ),
    ],
    ## Optional:
    # homepage="",
    # contributors=[
    #    sp.ContributorProperties(
    #        ## Required:
    #        title="",
    #        ## Optional:
    #        path="",
    #        email="",
    #        given_name="",
    #        family_name="",
    #        organization="",
    #        roles=[""],
    #    ),
    # ],
    # keywords=[""],
    # image="",
    # sources=[
    #    sp.SourceProperties(
    #        ## Required:
    #        title="",
    #        ## Optional:
    #        path="",
    #        email="",
    #        version="",
    #    ),
    # ],
    # resources=[
    #     resource_properties,
    # ],
    ## Autogenerated:
    id="{{ properties.id }}",
    version="{{ properties.version }}",
    created="{{ properties.created }}",
)
