# {{ properties.title or "N/A" }}

{{ properties.description or "N/A" }}

| Field    | Value                                   |
|----------|-----------------------------------------|
| Name     | {{ properties.name | inline_code }}     |
| ID       | {{ properties.id | inline_code }}       |
| Version  | {{ properties.version | inline_code }}  |
| Homepage | {{ properties.homepage | format_link }} |
| Created  | {{ properties.created | format_date }}  |
| Licenses | {{ properties.licenses | join_names }}  |

{% if properties.resources -%}
## Data Resources
{% for resource in properties.resources %}
### {{ resource.title }}

{{ resource.description }}
{% if resource.schema.fields %}
| Name | Title | Type | Description |
|------|-------|------|-------------|
{%- for field in resource.schema.fields %}
| {{ field.name | inline_code }} | {{ field.title }} | {{ field.type | inline_code }} | {{ field.description }} |
{%- endfor %}

: Resource fields in {{ resource.title }} and some details about them.
{% endif %}
{%- endfor -%}
{% else -%}
No resources available.
{% endif %}
