# ruff: noqa
# mypy: ignore-errors
email  # unused variable (src/seedcase_sprout/properties.py:91)
given_name  # unused variable (src/seedcase_sprout/properties.py:92)
family_name  # unused variable (src/seedcase_sprout/properties.py:93)
organization  # unused variable (src/seedcase_sprout/properties.py:94)
roles  # unused variable (src/seedcase_sprout/properties.py:95)
email  # unused variable (src/seedcase_sprout/properties.py:137)
reference  # unused variable (src/seedcase_sprout/properties.py:173)
label  # unused variable (src/seedcase_sprout/properties.py:186)
required  # unused variable (src/seedcase_sprout/properties.py:242)
pattern  # unused variable (src/seedcase_sprout/properties.py:244)
enum  # unused variable (src/seedcase_sprout/properties.py:245)
min_length  # unused variable (src/seedcase_sprout/properties.py:246)
max_length  # unused variable (src/seedcase_sprout/properties.py:247)
minimum  # unused variable (src/seedcase_sprout/properties.py:248)
maximum  # unused variable (src/seedcase_sprout/properties.py:249)
exclusive_minimum  # unused variable (src/seedcase_sprout/properties.py:250)
exclusive_maximum  # unused variable (src/seedcase_sprout/properties.py:251)
json_schema  # unused variable (src/seedcase_sprout/properties.py:252)
example  # unused variable (src/seedcase_sprout/properties.py:287)
constraints  # unused variable (src/seedcase_sprout/properties.py:288)
categories  # unused variable (src/seedcase_sprout/properties.py:289)
categories_ordered  # unused variable (src/seedcase_sprout/properties.py:290)
fields_match  # unused variable (src/seedcase_sprout/properties.py:332)
unique_keys  # unused variable (src/seedcase_sprout/properties.py:334)
foreign_keys  # unused variable (src/seedcase_sprout/properties.py:335)
missing_values  # unused variable (src/seedcase_sprout/properties.py:336)
mediatype  # unused variable (src/seedcase_sprout/properties.py:391)
encoding  # unused variable (src/seedcase_sprout/properties.py:392)
bytes  # unused variable (src/seedcase_sprout/properties.py:393)
error_type  # unused variable (src/seedcase_sprout/examples.py:395)
error  # unused variable (src/seedcase_sprout/examples.py:396)
traceback  # unused variable (src/seedcase_sprout/examples.py:397)
none  # unused variable (tests/test_get_nested_attr.py:7)
Number_1  # unused variable (tests/test_get_nested_attr.py:8)
